#!/bin/bash
#
# Copyright (c) 2021,2022 kz
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# * Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice, 
#   this list of conditions and the following disclaimer in the documentation 
#   and/or other materials provided with the distribution.
# * Neither the name of the <organization> nor the names of its contributors 
#   may be used to endorse or promote products derived from this software 
#   without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

IMAGE_NAME=recorder:latest
CONTAINER_NAME=recorder_`date "+%Y%m%d"`
HOST_NAME=rec
HOST_DATA_ROOT=$HOME/recorder
CONTAINER_CHINACHU_ROOT=/home/chinachu/Chinachu

if [ $# -lt 1 ] || [ ! -d $1 ] || [ ! -w $1 ]; then
  echo
  echo "Usage `basename $0` writable_data_directory"
  echo

  exit 1
else
  HOST_DATA_ROOT=$1
fi

PX4_DEVICES=`ls /dev/px4video*`
if [ -z "$PX4_DEVICES" ]; then
  echo "There're no px4 devices"
  echo
  exit 1
fi


# docker setup
if [ ! -f /usr/bin/docker ]; then
  sudo apt update
  sudo apt -y install apt-transport-https ca-certificates curl gnupg lsb-release
  curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /usr/share/keyrings/docker-archive-keyring.gpg
  echo "deb [arch=amd64 signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
  sudo apt update
  sudo apt -y install docker-ce docker-ce-cli containerd.io
  sudo chmod +x /usr/local/bin/docker-compose
  sudo gpasswd -a $USER docker
fi

# nodejs version constrained by Mirakurun
sudo apt install jq
NODE_VER=$(curl https://raw.githubusercontent.com/Chinachu/Mirakurun/master/package.json | jq -r '.engines.node' | sed -e "s/.*[^0-9.]//g")

docker ps -a | grep -q $CONTAINER_NAME
if [ $? -eq 0 ]; then
  echo "$CONTAINER_NAME has already been started"
  echo
  exit 1
fi


# build recorder image
docker build . -f Dockerfile \
           --build-arg UBUNTU_VERSION=$(lsb_release -r | awk '{print $2}') \
           --build-arg CHINACHU_ROOT=$CONTAINER_CHINACHU_ROOT \
           --build-arg CHINACHU_UID=$(id -u) \
           --build-arg CHINACHU_GID=$(id -g) \
           --build-arg NODE_VER=$NODE_VER \
           -t $IMAGE_NAME
if [ $? -ne 0 ]; then
  echo "Failed to build docker image"
  echo
  exit 1
fi


# preparing initial config files

mkdir -p $HOST_DATA_ROOT/recorded
mkdir -p $HOST_DATA_ROOT/chinachu/config
mkdir -p $HOST_DATA_ROOT/chinachu/data
mkdir -p $HOST_DATA_ROOT/mirakurun/config

## chinachu
TMP_CONTAINER=$(docker create $IMAGE_NAME)
if [ ! -f $HOST_DATA_ROOT/chinachu/config/config.json ]; then
  docker cp $TMP_CONTAINER:$CONTAINER_CHINACHU_ROOT/config.json $HOST_DATA_ROOT/chinachu/config/
fi
if [ ! -f $HOST_DATA_ROOT/chinachu/config/rules.json ]; then
  docker cp $TMP_CONTAINER:$CONTAINER_CHINACHU_ROOT/rules.json $HOST_DATA_ROOT/chinachu/config/
fi
docker rm $TMP_CONTAINER

for d in $PX4_DEVICES; do
  DOCKER_DEVICES="$DOCKER_DEVICES --device=$d"
done

## tuners.yml for mirakurun
if [ ! -f $HOST_DATA_ROOT/mirakurun/config/tuners.yml ]; then
  for d in $PX4_DEVICES; do
    TUNER_NUM=$(echo $d | sed -e 's|^/dev/px4video\([0-9]*\)$|\1|g')
    case $TUNER_NUM in
      0 | 1 | 4 | 5 | 8  | 9  | 12 | 13 | 16 | 17 | 20 | 21 )
        TYPE=$(echo -e "    - BS\n    - CS")
        NAME=PX4-S$(expr $TUNER_NUM + 1 - $TUNER_NUM / 2)
        ;;
      2 | 3 | 6 | 7 | 10 | 11 | 14 | 15 | 18 | 19 | 22 | 24 )
        TYPE=$(echo -e "    - GR")
        NAME=PX4-T$(expr $TUNER_NUM / 2 + $TUNER_NUM % 2)
        ;;
    esac

    cat <<EOT >> $HOST_DATA_ROOT/mirakurun/config/tuners.yml
- name: $NAME
  types:
$TYPE
  command: recpt1 --device $d <channel> - -
  decoder: arib-b25-stream-test
  isDisabled: false

EOT
  done
fi

docker run -d \
        $DOCKER_DEVICES \
        -v /var/run/pcscd/pcscd.comm:/var/run/pcscd/pcscd.comm \
        -v $HOST_DATA_ROOT/recorded:$CONTAINER_CHINACHU_ROOT/recorded \
        -v $HOST_DATA_ROOT/chinachu/config/config.json:$CONTAINER_CHINACHU_ROOT/config.json \
        -v $HOST_DATA_ROOT/chinachu/config/rules.json:$CONTAINER_CHINACHU_ROOT/rules.json \
        -v $HOST_DATA_ROOT/chinachu/data:$CONTAINER_CHINACHU_ROOT/data \
        -v $HOST_DATA_ROOT/mirakurun/config:/usr/local/etc/mirakurun \
        -v $HOST_DATA_ROOT/mirakurun/db:/usr/local/var/db/mirakurun \
        -v /etc/localtime:/etc/localtime:ro \
        -p 40772:40772 \
        -p 80:20772 \
        -h $HOST_NAME \
        --restart=always \
        --cap-add=sys_nice \
        --cap-add=sys_admin \
        --name $CONTAINER_NAME \
        -it $IMAGE_NAME

if [ $? != 0 ]; then
  echo "docker run failed... (removing the container)"

  docker rm $CONTAINER_NAME

  exit
fi
